@@ -10,6 +10,7 @@
 #include <glib/gstdio.h>
 
 #include <telepathy-glib/debug.h>
+#include <telepathy-glib/debug-sender.h>
 
 #include "debug.h"
 
@@ -67,17 +68,76 @@ gboolean debug_flag_is_set (DebugFlags flag)
   return flag & flags;
 }
 
+static GHashTable *flag_to_keys = NULL;
+
+static const gchar *
+debug_flag_to_domain (DebugFlags flag)
+{
+  if (G_UNLIKELY (flag_to_keys == NULL))
+    {
+      guint i;
+
+      flag_to_keys = g_hash_table_new_full (g_direct_hash, g_direct_equal,
+          NULL, g_free);
+
+      for (i = 0; keys[i].value; i++)
+        {
+          GDebugKey key = (GDebugKey) keys[i];
+          gchar *val;
+
+          val = g_strdup_printf ("%s/%s", G_LOG_DOMAIN, key.key);
+          g_hash_table_insert (flag_to_keys,
+              GUINT_TO_POINTER (key.value), val);
+        }
+    }
+
+  return g_hash_table_lookup (flag_to_keys, GUINT_TO_POINTER (flag));
+}
+
+void
+debug_free (void)
+{
+  if (flag_to_keys == NULL)
+    return;
+
+  g_hash_table_destroy (flag_to_keys);
+  flag_to_keys = NULL;
+}
+
+static void
+log_to_debug_sender (DebugFlags flag,
+    const gchar *message)
+{
+  TpDebugSender *dbg;
+  GTimeVal now;
+
+  dbg = tp_debug_sender_dup ();
+
+  g_get_current_time (&now);
+
+  tp_debug_sender_add_message (dbg, &now, debug_flag_to_domain (flag),
+      G_LOG_LEVEL_DEBUG, message);
+
+  g_object_unref (dbg);
+}
+
 void debug (DebugFlags flag,
                    const gchar *format,
                    ...)
 {
+  gchar *message;
+  va_list args;
+
+  va_start (args, format);
+  message = g_strdup_vprintf (format, args);
+  va_end (args);
+
+  log_to_debug_sender (flag, message);
+
   if (flag & flags)
-    {
-      va_list args;
-      va_start (args, format);
-      g_logv (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, format, args);
-      va_end (args);
-    }
+    g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "%s", message);
+
+  g_free (message);
 }
 
 #endif /* ENABLE_DEBUG */
