@@ -23,6 +23,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "salut-caps-channel-manager.h"
 #include "salut-connection.h"
 #include "salut-contact-channel.h"
 #include "salut-contact-manager.h"
@@ -51,7 +52,8 @@ _contact_finalized_cb (gpointer data, GObject *old_object);
 G_DEFINE_TYPE_WITH_CODE(SalutContactManager, salut_contact_manager,
     G_TYPE_OBJECT,
     G_IMPLEMENT_INTERFACE (TP_TYPE_CHANNEL_MANAGER,
-      salut_contact_manager_manager_iface_init));
+      salut_contact_manager_manager_iface_init);
+    G_IMPLEMENT_INTERFACE (SALUT_TYPE_CAPS_CHANNEL_MANAGER, NULL));
 
 enum
 {
