@@ -45,6 +45,7 @@ struct _SalutSelfClass {
     /* public abstract methods */
     gboolean (*announce) (SalutSelf *self, gint port, GError **error);
     gboolean (*set_presence) (SalutSelf *self, GError **error);
+    gboolean (*set_caps) (SalutSelf *self, GError **error);
     gboolean (*set_alias) (SalutSelf *self, GError **error);
     gboolean (*set_avatar) (SalutSelf *self, guint8 *data, gsize size,
         GError **error);
@@ -74,6 +75,10 @@ struct _SalutSelf {
     TpHandle olpc_cur_act_room;
     gchar *olpc_color;
 #endif
+    GHashTable *per_channel_manager_caps;
+    gchar *node;
+    gchar *hash;
+    gchar *ver;
 
     /* private */
     SalutConnection *connection;
@@ -107,6 +112,9 @@ gboolean salut_self_announce (SalutSelf *self, gint port, GError **error);
 gboolean salut_self_set_presence (SalutSelf *self,
     SalutPresenceId status, const gchar *message, GError **error);
 
+gboolean salut_self_set_caps (SalutSelf *self, const gchar *node,
+    const gchar *hash, const gchar *ver, GError **error);
+
 gboolean salut_self_set_avatar (SalutSelf *self, guint8 *data,
     gsize size, GError **error);
 
@@ -140,6 +148,8 @@ void salut_self_olpc_augment_invitation (SalutSelf *self,
     TpHandle room, TpHandle contact, GibberXmppNode *invite_node);
 #endif
 
+GSList * salut_self_get_features (SalutSelf *self);
+
 /* protected methods */
 void salut_self_established (SalutSelf *self);
 
