@@ -123,6 +123,8 @@ struct _TubesCapabilities
    * gchar *ServiceName -> Feature *feature
    */
   GHashTable *dbus_tube_caps;
+
+  gboolean tubes_supported;
 };
 
 static void
@@ -819,7 +821,8 @@ static const gchar * const old_tubes_channel_allowed_properties[] = {
 
 static const gchar * const stream_tube_channel_allowed_properties[] = {
     TP_IFACE_CHANNEL ".TargetHandle",
-    SALUT_IFACE_CHANNEL_TYPE_STREAM_TUBE ".Service",
+    TP_IFACE_CHANNEL ".TargetID",
+    TP_IFACE_CHANNEL_TYPE_STREAM_TUBE ".Service",
     NULL
 };
 
@@ -827,7 +830,8 @@ static const gchar * const stream_tube_channel_allowed_properties[] = {
 #if 0
 static const gchar * const dbus_tube_channel_allowed_properties[] = {
     TP_IFACE_CHANNEL ".TargetHandle",
-    SALUT_IFACE_CHANNEL_TYPE_DBUS_TUBE ".ServiceName",
+    TP_IFACE_CHANNEL ".TargetID",
+    TP_IFACE_CHANNEL_TYPE_DBUS_TUBE ".ServiceName",
     NULL
 };
 #endif
@@ -864,7 +868,7 @@ salut_tubes_manager_foreach_channel_class (
       (GDestroyNotify) tp_g_value_slice_free);
 
   value = tp_g_value_slice_new (G_TYPE_STRING);
-  g_value_set_static_string (value, SALUT_IFACE_CHANNEL_TYPE_STREAM_TUBE);
+  g_value_set_static_string (value, TP_IFACE_CHANNEL_TYPE_STREAM_TUBE);
   g_hash_table_insert (table, TP_IFACE_CHANNEL ".ChannelType",
       value);
 
@@ -885,7 +889,7 @@ salut_tubes_manager_foreach_channel_class (
       (GDestroyNotify) tp_g_value_slice_free);
 
   value = tp_g_value_slice_new (G_TYPE_STRING);
-  g_value_set_static_string (value, SALUT_IFACE_CHANNEL_TYPE_DBUS_TUBE);
+  g_value_set_static_string (value, TP_IFACE_CHANNEL_TYPE_DBUS_TUBE);
   g_hash_table_insert (table, TP_IFACE_CHANNEL ".ChannelType",
       value);
 
@@ -926,8 +930,8 @@ salut_tubes_manager_requestotron (SalutTubesManager *self,
 
   if (tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_TUBES) &&
   /* Temporarily disabled since the implementation is incomplete. */
-  /*  tp_strdiff (channel_type, SALUT_IFACE_CHANNEL_TYPE_DBUS_TUBE) && */
-      tp_strdiff (channel_type, SALUT_IFACE_CHANNEL_TYPE_STREAM_TUBE))
+  /*  tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_DBUS_TUBE) && */
+      tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_STREAM_TUBE))
     return FALSE;
 
   if (!tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_TUBES))
@@ -938,7 +942,7 @@ salut_tubes_manager_requestotron (SalutTubesManager *self,
               &error))
         goto error;
     }
-  else if (!tp_strdiff (channel_type, SALUT_IFACE_CHANNEL_TYPE_STREAM_TUBE))
+  else if (!tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_STREAM_TUBE))
     {
       if (tp_channel_manager_asv_has_unknown_properties (request_properties,
               tubes_channel_fixed_properties,
@@ -948,18 +952,18 @@ salut_tubes_manager_requestotron (SalutTubesManager *self,
 
       /* "Service" is a mandatory, not-fixed property */
       service = tp_asv_get_string (request_properties,
-                SALUT_IFACE_CHANNEL_TYPE_STREAM_TUBE ".Service");
+                TP_IFACE_CHANNEL_TYPE_STREAM_TUBE ".Service");
       if (service == NULL)
         {
           g_set_error (&error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
               "StreamTube requests must include '%s'",
-              SALUT_IFACE_CHANNEL_TYPE_STREAM_TUBE ".Service");
+              TP_IFACE_CHANNEL_TYPE_STREAM_TUBE ".Service");
           goto error;
         }
     }
 /* Temporarily disabled since the implementation is incomplete. */
 #if 0
-  else if (!tp_strdiff (channel_type, SALUT_IFACE_CHANNEL_TYPE_DBUS_TUBE))
+  else if (!tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_DBUS_TUBE))
     {
       GError *err = NULL;
 
@@ -971,12 +975,12 @@ salut_tubes_manager_requestotron (SalutTubesManager *self,
 
       /* "ServiceName" is a mandatory, not-fixed property */
       service = tp_asv_get_string (request_properties,
-                SALUT_IFACE_CHANNEL_TYPE_DBUS_TUBE ".ServiceName");
+                TP_IFACE_CHANNEL_TYPE_DBUS_TUBE ".ServiceName");
       if (service == NULL)
         {
           g_set_error (&error, TP_ERRORS, TP_ERROR_NOT_IMPLEMENTED,
               "Request missed a mandatory property '%s'",
-              SALUT_IFACE_CHANNEL_TYPE_DBUS_TUBE ".ServiceName");
+              TP_IFACE_CHANNEL_TYPE_DBUS_TUBE ".ServiceName");
           goto error;
         }
 
@@ -1059,7 +1063,7 @@ salut_tubes_manager_requestotron (SalutTubesManager *self,
         }
 
       new_channel = salut_tubes_channel_tube_request (tubes_channel,
-          channel_type, service);
+          request_token, request_properties, require_new);
       g_assert (new_channel != NULL);
 
       if (request_token != NULL)
@@ -1166,10 +1170,10 @@ add_service_to_array (gchar *service,
   channel_type_value = tp_g_value_slice_new (G_TYPE_STRING);
   if (type == TP_TUBE_TYPE_STREAM)
     g_value_set_static_string (channel_type_value,
-        SALUT_IFACE_CHANNEL_TYPE_STREAM_TUBE);
+        TP_IFACE_CHANNEL_TYPE_STREAM_TUBE);
   else
     g_value_set_static_string (channel_type_value,
-        SALUT_IFACE_CHANNEL_TYPE_DBUS_TUBE);
+        TP_IFACE_CHANNEL_TYPE_DBUS_TUBE);
   g_hash_table_insert (fixed_properties, TP_IFACE_CHANNEL ".ChannelType",
       channel_type_value);
 
@@ -1182,11 +1186,11 @@ add_service_to_array (gchar *service,
   g_value_set_string (target_handle_type_value, service);
   if (type == TP_TUBE_TYPE_STREAM)
     g_hash_table_insert (fixed_properties,
-        SALUT_IFACE_CHANNEL_TYPE_STREAM_TUBE ".Service",
+        TP_IFACE_CHANNEL_TYPE_STREAM_TUBE ".Service",
         target_handle_type_value);
   else
     g_hash_table_insert (fixed_properties,
-        SALUT_IFACE_CHANNEL_TYPE_DBUS_TUBE ".ServiceName",
+        TP_IFACE_CHANNEL_TYPE_DBUS_TUBE ".ServiceName",
         target_handle_type_value);
 
   dbus_g_type_struct_set (&monster,
@@ -1199,6 +1203,76 @@ add_service_to_array (gchar *service,
   g_ptr_array_add (arr, g_value_get_boxed (&monster));
 }
 
+static void
+add_generic_tube_caps (GPtrArray *arr)
+{
+  GValue monster1 = {0,};
+  GHashTable *fixed_properties;
+  GValue *channel_type_value;
+  GValue *target_handle_type_value;
+
+  /* StreamTube */
+  g_value_init (&monster1, TP_STRUCT_TYPE_REQUESTABLE_CHANNEL_CLASS);
+  g_value_take_boxed (&monster1,
+      dbus_g_type_specialized_construct (
+        TP_STRUCT_TYPE_REQUESTABLE_CHANNEL_CLASS));
+
+  fixed_properties = g_hash_table_new_full (g_str_hash, g_str_equal,
+      NULL, (GDestroyNotify) tp_g_value_slice_free);
+
+  channel_type_value = tp_g_value_slice_new (G_TYPE_STRING);
+  g_value_set_static_string (channel_type_value,
+      TP_IFACE_CHANNEL_TYPE_STREAM_TUBE);
+
+  g_hash_table_insert (fixed_properties, TP_IFACE_CHANNEL ".ChannelType",
+      channel_type_value);
+
+  target_handle_type_value = tp_g_value_slice_new (G_TYPE_UINT);
+  g_value_set_uint (target_handle_type_value, TP_HANDLE_TYPE_CONTACT);
+  g_hash_table_insert (fixed_properties,
+      TP_IFACE_CHANNEL ".TargetHandleType", target_handle_type_value);
+
+  dbus_g_type_struct_set (&monster1,
+      0, fixed_properties,
+      1, stream_tube_channel_allowed_properties,
+      G_MAXUINT);
+
+  g_hash_table_destroy (fixed_properties);
+  g_ptr_array_add (arr, g_value_get_boxed (&monster1));
+
+  /* FIXME: enable once D-Bus tube new API are implemented */
+#if 0
+  /* DBusTube */
+  g_value_init (&monster2, TP_STRUCT_TYPE_REQUESTABLE_CHANNEL_CLASS);
+  g_value_take_boxed (&monster2,
+      dbus_g_type_specialized_construct (
+        TP_STRUCT_TYPE_REQUESTABLE_CHANNEL_CLASS));
+
+  fixed_properties = g_hash_table_new_full (g_str_hash, g_str_equal,
+      NULL, (GDestroyNotify) tp_g_value_slice_free);
+
+  channel_type_value = tp_g_value_slice_new (G_TYPE_STRING);
+  g_value_set_static_string (channel_type_value,
+      SALUT_IFACE_CHANNEL_TYPE_DBUS_TUBE);
+
+  g_hash_table_insert (fixed_properties, TP_IFACE_CHANNEL ".ChannelType",
+      channel_type_value);
+
+  target_handle_type_value = tp_g_value_slice_new (G_TYPE_UINT);
+  g_value_set_uint (target_handle_type_value, TP_HANDLE_TYPE_CONTACT);
+  g_hash_table_insert (fixed_properties,
+      TP_IFACE_CHANNEL ".TargetHandleType", target_handle_type_value);
+
+  dbus_g_type_struct_set (&monster2,
+      0, fixed_properties,
+      1, gabble_tube_dbus_channel_get_allowed_properties (),
+      G_MAXUINT);
+
+  g_hash_table_destroy (fixed_properties);
+  g_ptr_array_add (arr, g_value_get_boxed (&monster2));
+#endif
+}
+
 static void
 salut_tubes_manager_get_contact_caps (
     SalutCapsChannelManager *manager,
@@ -1246,6 +1320,11 @@ salut_tubes_manager_get_contact_caps (
   if (caps == NULL)
     return;
 
+  if (!caps->tubes_supported)
+    return;
+
+  add_generic_tube_caps (arr);
+
   stream_tube_caps = caps->stream_tube_caps;
   dbus_tube_caps = caps->dbus_tube_caps;
 
@@ -1308,6 +1387,12 @@ _parse_caps_item (GibberXmppNode *node, gpointer user_data)
   if (NULL == var)
     return TRUE;
 
+  if (!g_str_has_prefix (var, GIBBER_TELEPATHY_NS_TUBES))
+    return TRUE;
+
+  /* tubes generic cap or service specific */
+  caps->tubes_supported = TRUE;
+
   if (g_str_has_prefix (var, GIBBER_TELEPATHY_NS_TUBES "/"))
     {
       /* http://telepathy.freedesktop.org/xmpp/tubes/$type#$service */
@@ -1378,6 +1463,8 @@ salut_tubes_manager_copy_caps (
   g_hash_table_foreach (caps_in->dbus_tube_caps, copy_caps_helper,
       caps_out->dbus_tube_caps);
 
+  caps_out->tubes_supported = caps_in->tubes_supported;
+
   *specific_caps_out = caps_out;
 }
 
@@ -1433,6 +1520,9 @@ salut_tubes_manager_caps_diff (
   if (old_caps == NULL || new_caps == NULL)
     return TRUE;
 
+  if (old_caps->tubes_supported != new_caps->tubes_supported)
+    return TRUE;
+
   if (g_hash_table_size (old_caps->stream_tube_caps) !=
       g_hash_table_size (new_caps->stream_tube_caps))
     return TRUE;
@@ -1474,8 +1564,8 @@ salut_tubes_manager_add_cap (SalutCapsChannelManager *manager,
 
   /* this channel is not for this factory */
   if (tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_TUBES) &&
-      tp_strdiff (channel_type, SALUT_IFACE_CHANNEL_TYPE_STREAM_TUBE) &&
-      tp_strdiff (channel_type, SALUT_IFACE_CHANNEL_TYPE_DBUS_TUBE))
+      tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_STREAM_TUBE) &&
+      tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_DBUS_TUBE))
     return;
 
   if (tp_asv_get_uint32 (cap,
@@ -1508,23 +1598,27 @@ salut_tubes_manager_add_cap (SalutCapsChannelManager *manager,
     {
       caps = tubes_capabilities_new ();
       g_hash_table_insert (*per_channel_manager_caps, manager, caps);
+
+      if (handle == base->self_handle)
+        /* We always support generic tubes caps */
+        caps->tubes_supported = TRUE;
     }
 
-  if (!tp_strdiff (channel_type, SALUT_IFACE_CHANNEL_TYPE_STREAM_TUBE))
+  if (!tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_STREAM_TUBE))
     {
       Feature *feat = g_slice_new (Feature);
       gchar *service = g_strdup (tp_asv_get_string (cap,
-          SALUT_IFACE_CHANNEL_TYPE_STREAM_TUBE ".Service"));
+          TP_IFACE_CHANNEL_TYPE_STREAM_TUBE ".Service"));
       feat->feature_type = FEATURE_OPTIONAL;
       feat->ns = g_strdup_printf ("%s/stream#%s", GIBBER_TELEPATHY_NS_TUBES,
           service);
       g_hash_table_insert (caps->stream_tube_caps, service, feat);
     }
-  else if (!tp_strdiff (channel_type, SALUT_IFACE_CHANNEL_TYPE_DBUS_TUBE))
+  else if (!tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_DBUS_TUBE))
     {
       Feature *feat = g_slice_new (Feature);
       gchar *service = g_strdup (tp_asv_get_string (cap,
-          SALUT_IFACE_CHANNEL_TYPE_DBUS_TUBE ".ServiceName"));
+          TP_IFACE_CHANNEL_TYPE_DBUS_TUBE ".ServiceName"));
       feat->feature_type = FEATURE_OPTIONAL;
       feat->ns = g_strdup_printf ("%s/dbus#%s", GIBBER_TELEPATHY_NS_TUBES,
           service);
